@@ -37,6 +37,7 @@ class worker:
                        selfunc     = None,
                        inj_pdf     = None,
                        n_total_inj = None,
+                       weights_inj = None,
                        ):
         self.dim                  = bounds.shape[0]
         self.bounds               = bounds
@@ -47,6 +48,7 @@ class worker:
                                            selection_function = selfunc,
                                            injection_pdf      = inj_pdf,
                                            total_injections   = n_total_inj,
+                                           weights_inj        = weights_inj,
                                            prior_pars         = get_priors(self.bounds,
                                                                            samples      = events,
                                                                            std          = hier_sigma,
@@ -225,17 +227,20 @@ def main():
     selfunc     = None
     inj_pdf     = None
     n_total_inj = None
+    weights_inj = None
     duration    = 1.
     if options.selfunc_file is not None:
-        selfunc, inj_pdf, n_total_inj, duration = load_selection_function(options.selfunc_file,
-                                                                          par           = options.par,
-                                                                          far_threshold = options.far_threshold,
-                                                                          snr_threshold = options.snr_threshold,
-                                                                          )
+        selfunc, inj_pdf, n_total_inj, duration, weights_inj = load_selection_function(options.selfunc_file,
+                                                                                       par           = options.par,
+                                                                                       far_threshold = options.far_threshold,
+                                                                                       snr_threshold = options.snr_threshold,
+                                                                                       )
         if not callable(selfunc):
             # Keeping only the samples within bounds
-            inj_pdf = inj_pdf[np.where((np.prod(options.bounds[:,0] < selfunc, axis = 1) & np.prod(selfunc < options.bounds[:,1], axis = 1)))]
-            selfunc = selfunc[np.where((np.prod(options.bounds[:,0] < selfunc, axis = 1) & np.prod(selfunc < options.bounds[:,1], axis = 1)))]
+            idx         = np.where((np.prod(options.bounds[:,0] < selfunc, axis = 1) & np.prod(selfunc < options.bounds[:,1], axis = 1)))
+            inj_pdf     = inj_pdf[idx]
+            selfunc     = selfunc[idx]
+            weights_inj = weights_inj[idx]
     if options.include_dvdz and not callable(selfunc):
         raise Exception("The inclusion of dV/dz*(1+z)^{-1} is available only with a selection function approximant.")
     if options.include_dvdz:
@@ -316,6 +321,7 @@ def main():
                                         selfunc          = dec_selfunc,
                                         inj_pdf          = inj_pdf,
                                         n_total_inj      = n_total_inj,
+                                        weights_inj      = weights_inj,
                                         )
                           for _ in range(options.n_parallel)])
         
