@@ -1,13 +1,8 @@
 import numpy as np
-
+import matplotlib.pyplot as plt
+import warnings
 from numba import njit, prange
 from pathlib import Path
-
-import matplotlib.pyplot as plt
-from matplotlib import rcParams
-from distutils.spawn import find_executable
-
-from figaro.cumulative import fast_cumulative
 from figaro.exceptions import FIGAROException
 from figaro import plot_settings
 
@@ -172,11 +167,11 @@ def compute_entropy_single_draw(mixture, n_draws = 1e3, return_error = False):
     """
     samples = mixture.rvs(int(n_draws))
     logP    = mixture.logpdf(samples)
-    entropy = np.mean(-logP)/log2e
+    entropy = np.mean(-logP)*log2e
     if not return_error:
         return entropy
     else:
-        dentropy = np.std(-logP)/(np.sqrt(n_draws)*log2e)
+        dentropy = np.std(-logP)/(np.sqrt(n_draws)/log2e)
         return entropy, dentropy
 
 def compute_entropy(draws, n_draws = 1e3, return_error = False):
@@ -224,7 +219,7 @@ def entropy(draws, out_folder = '.', exp_entropy = None, name = 'event', n_draws
     if exp_entropy is not None:
         ax.axhline(exp_entropy, lw = 0.5, ls = '--', c = 'r')
     ax.set_xlabel('$N$')
-    ax.set_ylabel('$S(N)\ [\mathrm{bits}]$')
+    ax.set_ylabel('$S(N)\\ [\\mathrm{bits}]$')
     if show:
         plt.show()
     if save:
