@@ -8,6 +8,7 @@ import importlib
 from figaro.exceptions import FIGAROException
 from figaro.mixture import mixture
 from figaro.cosmology import Planck18, Planck15, dVdz_approx_planck18, dVdz_approx_planck15
+from figaro._spin_prior import prior_chieff_chip_isotropic, chi_effective_prior_from_isotropic_spins, prior_component_spins, spin_costilt_pdf
 from pathlib import Path
 from tqdm import tqdm
 
@@ -44,8 +45,8 @@ GW_par = {'m1'                 : 'mass_1_source',
           's2y'                : 'spin_2y',
           's1z'                : 'spin_1z',
           's2z'                : 'spin_2z',
-          's1'                 : 'spin_1',
-          's2'                 : 'spin_2',
+          's1'                 : 'a_1',
+          's2'                 : 'a_2',
           'phi_12'             : 'phi_12',
           'psi'                : 'psi',
           'cos_iota'           : 'cos_iota',
@@ -302,7 +303,7 @@ def _unpack_gw_posterior(event, par, cosmology, rdstate, n_samples = -1, wavefor
                         # Playground
                         data = f['posterior_samples']
                 MDC_flag = True
-            # GWTC-2, GWTC-3
+            # GWTC-2, GWTC-3, GWTC-4
             except:
                 MDC_flag = False
                 if waveform == 'combined':
@@ -314,10 +315,14 @@ def _unpack_gw_posterior(event, par, cosmology, rdstate, n_samples = -1, wavefor
                         try:
                             data = f['C01:Mixed']['posterior_samples']
                         except:
+                            # GWTC-4 v1
                             try:
-                                data = f['IMRPhenomXPHM']['posterior_samples']
+                                data = f['C00:Mixed']['posterior_samples']
                             except:
-                                data = f['SEOBNRv4PHM']['posterior_samples']
+                                try:
+                                    data = f['IMRPhenomXPHM']['posterior_samples']
+                                except:
+                                    data = f['SEOBNRv4PHM']['posterior_samples']
                 else:
                     if waveform == 'imr':
                         try:
@@ -545,12 +550,15 @@ def _prior_gw(par, samples, cosmology = 'Planck15', uniform_dVdz = True):
             prior /= omega.dDLdz(np.array(samples[GW_par['z']]))
         if 'log_z' in par:
             prior *= samples[GW_par['z']]
+    if 'chi_eff' in par:
+        if 'chi_p' in par:
+            prior *= prior_chieff_chip_isotropic(samples[GW_par['chi_eff']], samples[GW_par['chi_p']], samples[GW_par['q']])
+        else:
+            prior *= chi_effective_prior_from_isotropic_spins(samples[GW_par['chi_eff']], samples[GW_par['q']])
     # Mass prior (uniform in detector-frame component masses)
     n_mass_pars = np.sum([item in par for item in ['m1','m2','mc','q']])
     if n_mass_pars > 0:
         prior *= (1+np.array(samples[GW_par['z']]))**np.min([n_mass_pars, 2])
-    if 'q' in par:
-        prior *= np.array(samples[GW_par['m1']])/(1+np.array(samples[GW_par['z']]))
     if ('mc' in par or 'mc_detect' in par):
         prior *= (1 + np.array(samples[GW_par['q']])) ** 0.2 / np.array(samples[GW_par['q']]) ** 0.6
     return prior
@@ -720,21 +728,23 @@ def load_selection_function(file, par = None, far_threshold = 1, snr_threshold =
         selfunc           = selfunc_module.selection_function
         inj_pdf           = None
         n_total_inj       = None
+        weights           = None
         try:
             duration      = selfunc_module.duration
         except:
             duration      = 1.
     else:
-        if ext not in ['h5','hdf5']:
+        if ext not in ['h5','hdf','hdf5']:
             samples     = np.loadtxt(file)
             det_idx     = samples[:,-1]
             selfunc     = samples[:,:-2][det_idx == 1]
             inj_pdf     = samples[:,-2][det_idx == 1]
             n_total_inj = len(samples)
+            weights     = np.ones(len(samples))
             duration    = 1.
         else:
-            selfunc, inj_pdf, n_total_inj, duration = _unpack_injections(file, par, far_threshold, snr_threshold, cosmology)
-    return selfunc, inj_pdf, n_total_inj, duration
+            selfunc, inj_pdf, n_total_inj, duration, weights = _unpack_injections(file, par, far_threshold, snr_threshold, cosmology)
+    return selfunc, inj_pdf, n_total_inj, duration, weights
 
 def _unpack_injections(file, par, far_threshold = 1., snr_threshold = 10, cosmology = 'Planck15'):
     """
@@ -765,33 +775,53 @@ def _unpack_injections(file, par, far_threshold = 1., snr_threshold = 10, cosmol
         wrong_pars = [p for p in par if p not in loadable_inj_pars]
         raise FIGAROException("The following parameters are not implemented: "+", ".join(wrong_pars))
     with h5py.File(file, 'r') as f:
-        data          = f['injections']
-        joint_dataset = 'name' in data.keys()
-        n_total_inj   = int(data.attrs['total_generated'])
-        duration      = data.attrs['analysis_time_s']/(60.*60.*24.*365) # Years
         try:
-            far_idx = np.zeros(np.array(data['far_cwb']).shape, dtype = bool)
+            # O1, O2, O3 injections
+            data          = f['injections']
+            O4            = False
+            joint_dataset = 'name' in data.keys()
+            n_total_inj   = int(data.attrs['total_generated'])
+            duration      = data.attrs['analysis_time_s']/(60.*60.*24.*365) # Years
         except KeyError:
-            far_idx = np.zeros(np.array(data['snr']).shape, dtype = bool)
-        for key in data.keys():
-            if 'ifar' in key.lower():
-                far_idx |= data[key][()] > 1./far_threshold
-        if joint_dataset:
-            # O1+O2+O3
-            names = np.array(data['name'], dtype = str)
-            snr   = np.array(data['optimal_snr_net'])
-            idx   = np.where(names == 'o3', far_idx, snr > snr_threshold)
+            # O4 injections
+            data      = f['events']
+            O4        = True
+            n_total_inj   = int(f.attrs['total_generated'])
+            duration      = np.float64(f.attrs['total_analysis_time'])/(60.*60.*24.*365) # Years
+        if O4:
+            snr = data['semianalytic_observed_phase_maximized_snr_net'][:]
+            far = np.min([data['%s_far'%search][:] for search in f.attrs['searches']], axis=0)
+            idx = (snr > snr_threshold) | (far < far_threshold)
         else:
-            if np.sum(far_idx) > 0:
-                # O3 only
-                idx = np.where(far_idx, True, False)
+            try:
+                far_idx = np.zeros(np.array(data['far_cwb']).shape, dtype = bool)
+            except KeyError:
+                far_idx = np.zeros(np.array(data['snr']).shape, dtype = bool)
+            for key in data.keys():
+                if 'ifar' in key.lower():
+                    far_idx |= data[key][()] > 1./far_threshold
+            if joint_dataset:
+                # O1+O2+O3
+                names = np.array(data['name'], dtype = str)
+                snr   = np.array(data['optimal_snr_net'])
+                idx   = np.where(names == 'o3', far_idx, snr > snr_threshold)
             else:
-                # Simulated dataset (SNR filter)
-                snr = np.array(data['snr'])
-                idx = np.where(snr > snr_threshold, True, False)
+                if np.sum(far_idx) > 0:
+                    # O3 only
+                    idx = np.where(far_idx, True, False)
+                else:
+                    # Simulated dataset (SNR filter)
+                    snr = np.array(data['snr'])
+                    idx = np.where(snr > snr_threshold, True, False)
         # Load samples
         samples = np.zeros((len(par), np.sum(idx)))
         inj_pdf = np.ones(np.sum(idx))
+        if O4:
+            # O4
+            weights = np.array(data['weights'])[idx]
+        else:
+            # O1, O2, O3
+            weights = np.array(data['mixture_weight'])[idx]
         # Parameters
         m1  = np.array(data[inj_par['m1']])[idx]
         m2  = np.array(data[inj_par['m2']])[idx]
@@ -804,6 +834,10 @@ def _unpack_injections(file, par, far_threshold = 1., snr_threshold = 10, cosmol
             s2x = np.array(data[inj_par['s2x']])[idx]
             s2y = np.array(data[inj_par['s2y']])[idx]
             s2z = np.array(data[inj_par['s2z']])[idx]
+            s1  = np.sqrt(s1x**2+s1y**2+s1z**2)
+            s2  = np.sqrt(s2x**2+s2y**2+s2z**2)
+            cos_tilt_1 = s1z/s1
+            cos_tilt_2 = s2z/s2
         except:
             pass
         for i, lab in enumerate(par):
@@ -827,13 +861,19 @@ def _unpack_injections(file, par, far_threshold = 1., snr_threshold = 10, cosmol
                     samples[i] = m2*(1+z)
                 # Spins
                 if lab == 's1':
-                    samples[i] = np.sqrt(s1x**2+s1y**2+s1z**2)
+                    samples[i] = s1
                 if lab == 's2':
-                    samples[i] = np.sqrt(s2x**2+s2y**2+s2z**2)
+                    samples[i] = s2
                 if lab == 'chi_eff':
-                    samples[i] = (s1z + q*s2z)/(1+q)
+                    chi_eff    = (s1z + q*s2z)/(1+q)
+                    samples[i] = chi_eff
                 if lab == 'chi_p':
-                    samples[i] = np.maximum(np.sqrt(s1x**2+s1y**2), np.sqrt(s2x**2+s2y**2)*q*(4*q+3)/(4+3*q))
+                    chi_p      = np.maximum(np.sqrt(s1x**2+s1y**2), np.sqrt(s2x**2+s2y**2)*q*(4*q+3)/(4+3*q))
+                    samples[i] = chi_p
+                if lab == 'cos_tilt_1':
+                    samples[i] = cos_tilt_1
+                if lab == 'cos_tilt_2':
+                    samples[i] = cos_tilt_2
                 # Distance
                 if lab == 'luminosity_distance':
                     samples[i] = omega.LuminosityDistance(z)
@@ -843,13 +883,15 @@ def _unpack_injections(file, par, far_threshold = 1., snr_threshold = 10, cosmol
         n_mass_pars = len([lab for lab in par if lab in mass_parameters])
         n_det_pars  = len([lab for lab in par if lab in detector_parameters])
         n_spin_pars = len([lab for lab in par if lab in spin_parameters])
-        if joint_dataset:
+        if O4:
+            if not (('z' in par) and (n_mass_pars == 2)):
+                raise FIGAROException("Cannot unpack individual parameter sampling PDF for O4 injections")
+            log_inj_pdf = np.array(data['lnpdraw_mass1_source_mass2_source_redshift_spin1x_spin1y_spin1z_spin2x_spin2y_spin2z'])[idx]
+            inj_pdf     = np.exp(log_inj_pdf)
+        elif joint_dataset:
             if not (('z' in par) and (n_mass_pars == 2)):
                 raise FIGAROException("Cannot unpack individual parameter sampling PDF for combined injections")
             inj_pdf  = np.array(data['sampling_pdf'])[idx]
-            # Remove spins if not needed
-            spin_pdf = 1./(4*np.pi*(s1x**2+s1y**2+s1z**2))*1./(4*np.pi*(s2x**2+s2y**2+s2z**2))
-            inj_pdf /= spin_pdf**((6-n_spin_pars)/6.)
         else:
             inj_pdf = np.ones(np.sum(idx))
             # Masses
@@ -875,17 +917,27 @@ def _unpack_injections(file, par, far_threshold = 1., snr_threshold = 10, cosmol
                 inj_pdf *= np.array(data['right_ascension_sampling_pdf'])[idx]
             if 'dec' in par:
                 inj_pdf *= np.array(data['declination_sampling_pdf'])[idx]
+        # Spin priors
+        spin_pdf    = 4*np.pi**2*s1**2*s2**2
+        if n_spin_pars > 0:
+            if 'chi_eff' in par:
+                spin_pdf *=  4.
+                if not 'q' in par:
+                    raise FIGAROException("Effective spin requires mass ratio")
+                if 'chi_p' in par:
+                    spin_pdf *= prior_chieff_chip_isotropic(chi_eff, chi_p, q)
+                else:
+                    spin_pdf *= chi_effective_prior_from_isotropic_spins(chi_eff, q)
+            if not (('cos_tilt_1' in par) and ('cos_tilt_2' in par)):
+                spin_pdf /= spin_costilt_pdf(cos_tilt_1)*spin_costilt_pdf(cos_tilt_2)
+        else:
+            # Remove spin entirely
+            spin_pdf /= prior_component_spins(s1x, s1y, s1z, s2x, s2y, s2z)*4*np.pi**2*s1**2*s2**2
+        inj_pdf *= spin_pdf
         # Change of variable
         if 'q' in par:
             inj_pdf *= m1
-        if 's1' in par or 'chi_eff' in par or 'chi_p' in par:
-            inj_pdf *= 2*np.pi*(s1x**2+s1y**2+s1z**2)
-        if 's2' in par or 'chi_eff' in par or 'chi_p' in par:
-            inj_pdf *= 2*np.pi*(s2x**2+s2y**2+s2z**2)
         if n_det_pars > 0:
             inj_pdf /= (1+z)**n_det_pars
 
-#    import matplotlib.pyplot as plt
-#    plt.hist(np.log(inj_pdf))
-#    plt.show()
-    return samples.T, inj_pdf, n_total_inj, duration
+    return samples.T, inj_pdf, n_total_inj, duration, weights
