@@ -2,7 +2,8 @@ import numpy as np
 from numba import njit
 from scipy.special import logsumexp
 from figaro.exceptions import FIGAROException
-from figaro.likelihood import log_norm, inv_jit
+from figaro._likelihood import log_norm
+from figaro._numba_functions import inv_jit
 from figaro.decorators import probit
 
 @njit
@@ -42,14 +43,19 @@ def _marginalise(mix, axis = -1):
     # Circular import
     from figaro.mixture import mixture
     ax     = np.atleast_1d(axis)
+    if len(axis) == 0:
+        return mix
     dim    = mix.dim - len(ax)
     if dim < 1:
         raise FIGAROException("Cannot marginalise out all dimensions")
     means  = np.delete(mix.means, ax, axis = -1)
     covs   = np.delete(np.delete(mix.covs, ax, axis = -1), ax, axis = -2)
     bounds = np.delete(mix.bounds, ax, axis = 0)
-    
-    return mixture(means, covs, mix.w, bounds, dim, mix.n_cl, mix.n_pts, probit = mix.probit)
+    if mix.components is None:
+        make_comp = False
+    else:
+        make_comp = True
+    return mixture(means, covs, mix.w, bounds, dim, mix.n_cl, mix.n_pts, mix.alpha, probit = mix.probit, make_comp = make_comp, alpha_factor = mix.alpha_factor)
 
 def marginalise(draws, axis = -1):
     """
@@ -62,7 +68,7 @@ def marginalise(draws, axis = -1):
     Returns:
         figaro.mixture.mixture: the marginalised mixture(s)
     """
-    if axis == []:
+    if len(axis) == 0:
         return draws
     if np.iterable(draws):
         return np.array([_marginalise(d, axis) for d in draws])
@@ -116,7 +122,6 @@ def _condition(mix, vals, dims, norm = True, filter = True, tol = 1e-4):
         log_weights -= _marginalise(mix, axis = np.arange(mix.dim)[~idx])._logpdf_probit(vals)
     # Filter out components with negligible weights
     idx_filt = [True for _ in range(len(log_weights))]
-    norm_const = 0.
     if filter:
         ww = np.exp(log_weights)
         idx = np.argsort(ww)
@@ -124,7 +129,11 @@ def _condition(mix, vals, dims, norm = True, filter = True, tol = 1e-4):
         idx_filt = [i in idx[m:] for i in range(len(ww))]
         if norm:
             log_weights -= logsumexp(log_weights[idx_filt])
-    return mixture(means[idx_filt], covs[idx_filt], np.exp(log_weights[idx_filt]), bounds, dim, len(log_weights[idx_filt]), mix.n_pts, probit = mix.probit, log_w = log_weights[idx_filt])
+    if mix.components is None:
+        make_comp = False
+    else:
+        make_comp = True
+    return mixture(means[idx_filt], covs[idx_filt], np.exp(log_weights[idx_filt]), bounds, dim, len(log_weights[idx_filt]), mix.n_pts, mix.alpha, probit = mix.probit, log_w = log_weights[idx_filt], make_comp = make_comp, alpha_factor = mix.alpha_factor)
 
 def condition(draws, vals, dims, norm = True, filter = True, tol = 1e-4):
     """
