@@ -1165,6 +1165,7 @@ class HDPGMM(DPGMM):
                        selection_function = None,
                        injection_pdf      = None,
                        total_injections   = None,
+                       weights_inj        = None,
                        lower_limit_alpha  = 1e-3,
                        ):
         bounds   = np.atleast_2d(bounds)
@@ -1185,23 +1186,32 @@ class HDPGMM(DPGMM):
         self.lower_limit_alpha = lower_limit_alpha
         if not callable(self.selfunc) and self.selfunc is not None:
             try:
-                self.log_inj_pdf = np.log(injection_pdf)
-                self.total_inj   = int(total_injections)
+                self.log_inj_pdf    = np.log(injection_pdf)
+                self.np_log_inj_pdf = np.copy(np.log(injection_pdf))
+                self.total_inj      = int(total_injections)
+                if weights_inj is not None:
+                    self.weights_inj = weights_inj
+                else:
+                    self.weights_inj = np.ones(len(self.log_inj_pdf))
             except TypeError:
                 raise FIGAROException("Please provide injection pdf")
             self.selfunc_probit   = self.selfunc
             self.log_jacobian_inj = np.zeros(len(self.selfunc))
             if self.probit:
-                self.selfunc_probit   = transform_to_probit(self.selfunc, self.bounds)
+                self.selfunc_probit   = np.atleast_2d(transform_to_probit(self.selfunc, self.bounds))
+                if self.selfunc_probit.shape[0] == 1:
+                    self.selfunc_probit = self.selfunc_probit.T
                 self.log_jacobian_inj = -probit_logJ(self.selfunc_probit, self.bounds, self.probit)
         # MC samples
         self._draw_MC_samples()
         
-    def initialise(self):
+    def initialise(self, prior_pars = None):
         """
         Initialise the mixture to initial conditions
         """
         super().initialise()
+        if prior_pars is not None:
+            self.exp_sigma, self.a = prior_pars
         self.evaluated_logL = {}
         self._draw_MC_samples()
     
@@ -1230,11 +1240,11 @@ class HDPGMM(DPGMM):
                         self.log_alpha_factor = np.array([np.log(np.mean(self.selfunc(mn(m,s, allow_singular = True).rvs(self.MC_draws*1)))) for m, s in zip(self.mu_MC, self.sigma_MC)])
             # Injections
             else:
-                self.log_alpha_factor = np.array([logsumexp(mn(m,s, allow_singular = True).logpdf(self.selfunc_probit) + self.log_jacobian_inj - self.log_inj_pdf) - np.log(self.total_inj) for m, s in zip(self.mu_MC, self.sigma_MC)])
+                self.log_alpha_factor = np.array([logsumexp(mn(m,s, allow_singular = True).logpdf(self.selfunc_probit) + self.log_jacobian_inj - self.log_inj_pdf, b = self.weights_inj) - np.log(self.total_inj) for m, s in zip(self.mu_MC, self.sigma_MC)])
                 # TODO: make more efficient
                 std = []
                 for m, s, a in zip(self.mu_MC, self.sigma_MC, self.log_alpha_factor):
-                    x = mn(m,s, allow_singular = True).logpdf(self.selfunc_probit) + self.log_jacobian_inj - self.log_inj_pdf
+                    x = self.weights_inj*(mn(m,s, allow_singular = True).logpdf(self.selfunc_probit) + self.log_jacobian_inj - self.log_inj_pdf)
 #                    std.append(np.sqrt(np.sum((np.exp(x)-np.exp(a))**2)/self.total_inj**2))
                     std.append(np.sqrt(np.sum(np.exp(2*x)/self.total_inj**2) - np.exp(2*a)/self.total_inj))
                 std = np.array(std)
@@ -1424,7 +1434,8 @@ class HDPGMM(DPGMM):
             log_w, w           = self.log_w, self.w
             self.log_w         = N_pts - logsumexp_jit(N_pts)
             self.w             = np.exp(self.log_w)
-            N_pts              = self.w*self.n_pts/self._compute_individual_alpha_factors(idx)
+            aa                 = self.compute_alpha_factor()#self._compute_individual_alpha_factors(idx)
+            N_pts              = (self.w/aa )*self.n_pts
             new_w              = dirichlet(N_pts+self.alpha/self.n_cl).rvs()[0]
             self.w             = new_w
             self.log_w         = np.log(new_w)
@@ -1454,7 +1465,7 @@ class HDPGMM(DPGMM):
                 alpha_factors = np.array([np.mean(self.selfunc(mn(comp.mu, comp.sigma, allow_singular = True).rvs(self.MC_draws*1))) for comp in np.array(self.mixture)[idx]])
         # Injections
         else:
-            alpha_factors = np.array([np.exp(logsumexp_jit(mn(comp.mu, comp.sigma, allow_singular = True).logpdf(self.selfunc_probit) + self.log_jacobian_inj - self.log_inj_pdf) - np.log(self.total_inj)) for comp in np.array(self.mixture)[idx]])
+            alpha_factors = np.array([np.exp(logsumexp(mn(comp.mu, comp.sigma, allow_singular = True).logpdf(self.selfunc_probit) + self.log_jacobian_inj - self.log_inj_pdf, b = self.weights_inj) - np.log(self.total_inj)) for comp in np.array(self.mixture)[idx]])
         return alpha_factors
     
     def compute_alpha_factor(self):
@@ -1469,10 +1480,10 @@ class HDPGMM(DPGMM):
             alpha_factor = np.mean(self.selfunc(self.rvs(self.MC_draws)))
         # Injections
         else:
-            alpha_factor = np.exp(logsumexp_jit(self.logpdf(self.selfunc) + self.log_jacobian_inj - self.log_inj_pdf) - np.log(self.total_inj))
+            alpha_factor = np.exp(logsumexp(self.logpdf(self.selfunc) + self.log_jacobian_inj - self.log_inj_pdf, b = self.weights_inj) - np.log(self.total_inj))
         return alpha_factor
 
-    def density_from_samples(self, events, make_comp = True):
+    def density_from_samples(self, events, make_comp = True, initialise = True):
         """
         Reconstruct the probability density from a set of samples.
         
@@ -1490,7 +1501,8 @@ class HDPGMM(DPGMM):
         for id in np.random.choice(self.n_pts, size = self.n_reassignments, replace = True):
             self._reassign_point(id)
         d = self.build_mixture(make_comp = make_comp)
-        self.initialise()
+        if initialise:
+            self.initialise()
         return d
 
     def _reassign_point(self, id):
