@@ -100,7 +100,7 @@ class PPPlot(axes.Axes):
         
 projection_registry.register(PPPlot)
 
-def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bounds = None, out_folder = '.', name = 'density', n_pts = 1000, label = None, unit = None, hierarchical = False, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = None, fig = None, colors = ['steelblue', 'darkturquoise', 'mediumturquoise']):
+def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bounds = None, out_folder = '.', name = 'density', n_pts = 1000, label = None, unit = None, hierarchical = False, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = None, fig = None, colors = ['steelblue', 'darkturquoise', 'mediumturquoise'], large_font = True):
     """
     Plot the recovered 1D distribution along with the injected distribution and samples from the true distribution (both if available).
     
@@ -216,14 +216,25 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
     if label is None:
         label = 'x'
     if unit is None or unit == '':
-        ax.set_xlabel('${0}$'.format(label))
+        if large_font:
+            ax.set_xlabel('${0}$'.format(label), fontsize = 20)
+        else:
+            ax.set_xlabel('${0}$'.format(label))
     else:
-        ax.set_xlabel('${0}\ [{1}]$'.format(label, unit))
-    ax.set_ylabel('$p({0})$'.format(label))
+        if large_font:
+            ax.set_xlabel('${0}\ [{1}]$'.format(label, unit), fontsize = 20)
+        else:
+            ax.set_xlabel('${0}\ [{1}]$'.format(label, unit))
+    if large_font:
+        ax.set_ylabel('$p({0})$'.format(label), fontsize = 20)
+    else:
+        ax.set_ylabel('$p({0})$'.format(label))
     if samples is not None:
         ax.set_xlim(xlim)
     ax.set_ylim(bottom = 1e-5, top = np.max(p[95])*1.1)
     ax.legend(loc = 0)
+    if large_font:
+        ax.tick_params(axis='both', which='both', labelsize=18)
     
     fig.align_labels()
     
@@ -290,13 +301,24 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
         if label is None:
             label = 'x'
         if unit is None or unit == '':
-            ax.set_xlabel('${0}$'.format(label))
+            if large_font:
+                ax.set_xlabel('${0}$'.format(label), fontsize = 20)
+            else:
+                ax.set_xlabel('${0}$'.format(label))
         else:
-            ax.set_xlabel('${0}\ [{1}]$'.format(label, unit))
-        ax.set_ylabel('$p({0})$'.format(label))
+            if large_font:
+                ax.set_xlabel('${0}\ [{1}]$'.format(label, unit), fontsize = 20)
+            else:
+                ax.set_xlabel('${0}\ [{1}]$'.format(label, unit))
+        if large_font:
+            ax.set_ylabel('$p({0})$'.format(label), fontsize = 20)
+        else:
+            ax.set_ylabel('$p({0})$'.format(label))
         ax.autoscale(True)
         ax.set_ylim(bottom = 1e-5, top = np.max(p[95])*1.1)
         ax.legend(loc = 0)
+        if large_font:
+            ax.tick_params(axis='both', which='both', labelsize=18)
         if save:
             fig.savefig(Path(log_folder, 'log_intrinsic_{0}.pdf'.format(name)), bbox_inches = 'tight')
             ax.set_yscale('linear')
@@ -555,7 +577,7 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
         plt.close()
     return fig
     
-def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', name = 'density', label = None, unit = None, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = '\mathrm{Median}', logx = False, logy = False, colors = ['steelblue','darkturquoise','mediumturquoise'], fig = None):
+def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', name = 'density', label = None, unit = None, show = False, save = True, subfolder = False, true_value = None, true_value_label = '\mathrm{True\ value}', injected_label = '\mathrm{Simulated}', median_label = '\mathrm{Median}', logx = False, logy = False, colors = ['steelblue','darkturquoise','mediumturquoise'], fig = None, large_font = True):
     """
     Plot a 1D distribution along with samples from the true distribution (if available).
     Differently from plot_median_cr, this method requires the distribution to be already evaluated.
@@ -628,14 +650,25 @@ def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', na
     if label is None:
         label = 'x'
     if unit is None or unit == '':
-        ax.set_xlabel('${0}$'.format(label))
+        if large_font:
+            ax.set_xlabel('${0}$'.format(label), fontsize = 20)
+        else:
+            ax.set_xlabel('${0}$'.format(label))
     else:
-        ax.set_xlabel('${0}\ [{1}]$'.format(label, unit))
-    ax.set_ylabel('$p({0})$'.format(label))
+        if large_font:
+            ax.set_xlabel('${0}\ [{1}]$'.format(label, unit), fontsize = 20)
+        else:
+            ax.set_xlabel('${0}\ [{1}]$'.format(label, unit))
+    if large_font:
+        ax.set_ylabel('$p({0})$'.format(label), fontsize = 20)
+    else:
+        ax.set_ylabel('$p({0})$'.format(label))
     if samples is not None:
         ax.set_xlim(xlim)
     ax.set_ylim(bottom = 1e-5, top = np.max(p[95])*1.1)
     ax.legend(loc = 0)
+    if large_font:
+        ax.tick_params(axis='both', which='both', labelsize=18)
     if logy:
         ax.set_yscale('log')
     if logx:
@@ -802,7 +835,7 @@ def pp_plot_levels(CR_levels, median_CR = None, out_folder = '.', name = 'MDC',
         plt.close()
     return fig
 
-def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, solid = False, overlap = 1., xlabel = None, ylabel = None, xunit = None, yunit = None, colormap = 'coolwarm', out_folder = '.', name = 'joyplot', subfolder = False, show = False, save = True, joy = False, colorbar = True, histogram = False):
+def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, solid = False, overlap = 1., xlabel = None, ylabel = None, xunit = None, yunit = None, colormap = 'coolwarm', out_folder = '.', name = 'joyplot', subfolder = False, show = False, save = True, joy = False, colorbar = True, histogram = False, large_font = True):
     """
     Make a joyplot (also known as ridgeline plot) of a set of distributions.
     Heavily inspired by leotac's JoyPy (https://github.com/leotac/joypy).
@@ -863,7 +896,11 @@ def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, so
         _axes.append(fig.add_subplot(gs[:, -1]))
         # Global colorbar
         cbar    = fig.colorbar(cmappable, cax = _axes[-1])
-        cbar.set_label(ylabel)
+        if large_font:
+            cbar.set_label(ylabel, fontsize = 20)
+            _axes[-1].tick_params(axis='both', which='both', labelsize=18)
+        else:
+            cbar.set_label(ylabel)
     if joy:
         [ax.set_facecolor('k') for ax in _axes]
     for i, dd in enumerate(draws[::-1]):
@@ -923,6 +960,8 @@ def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, so
     # so they have the same lims and ticks
     last_axis.set_xlim(_axes[0].get_xlim())
     last_axis.set_xticks(np.array(_axes[0].get_xticks()[1:-1]))
+    if large_font:
+        last_axis.tick_params(axis='both', which='both', labelsize=18)
     for t in last_axis.get_xticklabels():
         if not joy:
             t.set_visible(True)
@@ -939,7 +978,10 @@ def joyplot(draws, x_values, y_values, credible_regions = False, fill = True, so
     # Last axis on the back
     last_axis.zorder = min(a.zorder for a in _axes) - 1
     _axes = list(_axes) + [last_axis]
-    last_axis.set_xlabel(xlabel)
+    if large_font:
+        last_axis.set_xlabel(xlabel, fontsize = 20)
+    else:
+        last_axis.set_xlabel(xlabel)
 
     # The magic overlap happens here.
     h_pad = 5 + (-5*(1+overlap))
